<?php

/**
 * @file
 * Administration-related functions for Measuremail module.
 */

/**
 * Administration page form callback for the Measuremail module.
 *
 * @return array
 *   The admin form array.
 *
 * @ingroup forms
 */
function measuremail_subscription_admin($form, &$form_state) {
  $form = [];

  $form['measuremail_subscription'] = [
    '#type' => 'vertical_tabs',
  ];

  // ENDPOINT Settings.
  $form['measuremail_subscription_endpoint_data'] = [
    '#type' => 'fieldset',
    '#title' => t('Endpoint settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#group' => 'measuremail_subscription',
  ];
  $form['measuremail_subscription_endpoint_data']['measuremail_subscription_endpoint'] = [
    '#type' => 'textfield',
    '#title' => t('URL for service endpoint'),
    '#default_value' => variable_get('measuremail_subscription_endpoint', MEASUREMAIL_SUBSCRIPTION_ENDPOINT),
    '#description' => t('Enter the URL for the Measuremail endpoint (e.g: https://action.spike.email/{environment_number}/Subscriptions/Subscribe)'),
    '#required' => TRUE,
    '#maxlength' => 255,
  ];
  $form['measuremail_subscription_endpoint_data']['measuremail_subscription_id'] = [
    '#type' => 'textfield',
    '#title' => t('Subscription ID'),
    '#default_value' => variable_get('measuremail_subscription_id', MEASUREMAIL_SUBSCRIPTION_ID),
    '#description' => t('ID as set on "Subscriptions" > "Channels" > "Subscription ID"'),
    '#required' => TRUE,
    '#maxlength' => 255,
  ];

  // FIELDS Settings.
  $form['measuremail_subscription_form'] = [
    '#type' => 'fieldset',
    '#title' => t('Form Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#group' => 'measuremail_subscription',
  ];
  $options = variable_get_value('measuremail_subscription_measuremail_fields', ['default' => '']);
  $form['measuremail_subscription_form']['measuremail_subscription_measuremail_fields'] = [
    '#type' => 'textarea',
    '#title' => t('Measuremail Fields'),
    '#default_value' => $options,
    '#description' => t('List all fields declared in Measuremail here. Enter one value per line, in the format <em>key|label|required</em>.<br/><br/>The <strong>key</strong> is the field code defined in Measuremail without <code>&lt;? ?&gt;</code>. The <strong>label</strong> will be a translatable %language string which is displayed in front of the field. The <strong>required</strong> is optional and will be used to mark form fields as required/mandatory. e.g.: <em>email|Email|required</em> or <em>name|Name</em><br/><br/>Email is mandatory so you don\'t need to set it here.', ['%language' => language_default('name')]),
    '#required' => FALSE,
  ];
  $form['measuremail_subscription_form']['measuremail_subscription_page_url'] = [
    '#type' => 'textfield',
    '#title' => t('Page URL'),
    '#default_value' => variable_get('measuremail_subscription_page_url', MEASUREMAIL_SUBSCRIPTION_PAGE_URL),
    '#description' => t('Enter the URL of the page where the form must be displayed.'),
    '#required' => TRUE,
  ];
  $form['measuremail_subscription_form']['measuremail_subscription_language_code'] = [
    '#type' => 'textfield',
    '#title' => t('Language field code'),
    '#default_value' => variable_get('measuremail_subscription_language_code', MEASUREMAIL_SUBSCRIPTION_LANGUAGE_CODE),
    '#description' => t("Enter the code of the field used to store the contact's language (without <code>&lt;? ?&gt;</code>). If you do not need to store the contact's language, you can leave this empty.<br/><br/>If you set this field, Language will be an hidden field with current page language as default value."),
    '#required' => FALSE,
  ];
  $form['measuremail_subscription_form']['measuremail_subscription_email_field'] = [
    '#type' => 'textfield',
    '#title' => t('Email field name'),
    '#default_value' => variable_get('measuremail_subscription_email_field', MEASUREMAIL_SUBSCRIPTION_EMAIL_FIELD),
    '#description' => t("Enter the code of the field used to store the contact's email."),
    '#required' => TRUE,
  ];
  $languages_enabled_options = [
    'nl' => t('Dutch'),
    'fr' => t('French'),
    'de' => t('German'),
    'en' => t('English'),
  ];
  $form['measuremail_subscription_form']['measuremail_subscription_languages_enabled'] = [
    '#type' => 'checkboxes',
    '#title' => t('Enabled languages'),
    '#options' => $languages_enabled_options,
    '#default_value' => variable_get('measuremail_subscription_languages_enabled', [
      'nl',
      'fr',
      'de',
      'en',
    ]),
    '#description' => t('Choose the languages on which to enable the subscription page. On disabled languages, the subscription page will link to the subscription page in enabled languages.'),
    '#required' => TRUE,
  ];
  $form['measuremail_subscription_form']['measuremail_subscription_submit_button'] = [
    '#type' => 'textfield',
    '#title' => t('Submit value'),
    '#default_value' => variable_get('measuremail_subscription_submit_button', MEASUREMAIL_SUBSCRIPTION_SUBMIT_BUTTON),
    '#description' => t('Enter the text value that should appear on the submit button.'),
    '#required' => TRUE,
  ];
  // GDPR Required Settings.
  $form['measuremail_subscription_gdpr'] = [
    '#type' => 'fieldset',
    '#title' => t('GDPR Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#group' => 'measuremail_subscription',
  ];
  $form['measuremail_subscription_gdpr']['measuremail_subscription_formversion'] = [
    '#type' => 'textfield',
    '#title' => t('Form Version'),
    '#default_value' => variable_get('measuremail_subscription_formversion', MEASUREMAIL_SUBSCRIPTION_FORMVERSION),
    '#description' => t('Measure mail subscription form version.'),
  ];
  $form['measuremail_subscription_gdpr']['measuremail_subscription_privacyurl'] = [
    '#type' => 'textfield',
    '#title' => t('Privacy Policy page Url'),
    '#default_value' => variable_get('measuremail_subscription_privacyurl', MEASUREMAIL_SUBSCRIPTION_PRIVACYURL),
    '#description' => t('Absolute link to the privacy policy URL.'),
  ];
  $form['measuremail_subscription_gdpr']['measuremail_subscription_privacyversion'] = [
    '#type' => 'textfield',
    '#title' => t('Privacy Policy Version'),
    '#default_value' => variable_get('measuremail_subscription_privacyversion', MEASUREMAIL_SUBSCRIPTION_PRIVACYVERSION),
  ];

  // MESSAGE Settings.
  $form['measuremail_subscription_message'] = [
    '#type' => 'fieldset',
    '#title' => t('Message Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#group' => 'measuremail_subscription',
  ];
  $callbacktype_enabled_options = [
    'newpage' => t('New Page'),
    'inlinemessage' => t('Inline Message, using default Drupal Message System'),
  ];
  $form['measuremail_subscription_message']['measuremail_subscription_callback_type'] = [
    '#type' => 'radios',
    '#title' => t('Callback type'),
    '#default_value' => variable_get('measuremail_subscription_callback_type', MEASUREMAIL_SUBSCRIPTION_CALLBACK_TYPE),
    '#description' => t('Select the action to take after the form submission.'),
    '#required' => TRUE,
    '#options' => $callbacktype_enabled_options,
  ];
  $form['measuremail_subscription_message']['measuremail_subscription_callback_url'] = [
    '#type' => 'textfield',
    '#title' => t('Callback Url'),
    '#default_value' => variable_get('measuremail_subscription_callback_url', MEASUREMAIL_SUBSCRIPTION_CALLBACK_URL),
    '#description' => t('This should be set if "New Page" is defined as a callback type.'),
    '#states' => [
      'visible' => [
        ':input[name="measuremail_subscription_callback_type"]' => ['value' => 'newpage'],
      ],
      'required' => [
        ':input[name="measuremail_subscription_callback_type"]' => ['value' => 'newpage'],
      ],
    ],
  ];
  $form['measuremail_subscription_message']['measuremail_subscription_message_success'] = [
    '#type' => 'textarea',
    '#title' => t('Success'),
    '#default_value' => variable_get('measuremail_subscription_message_success', MEASUREMAIL_SUBSCRIPTION_MESSAGE_SUCCESS),
    '#description' => t('Enter the message that will be displayed if everything went well.'),
    '#required' => TRUE,
  ];
  $form['measuremail_subscription_message']['measuremail_subscription_message_update'] = [
    '#type' => 'textarea',
    '#title' => t('Update'),
    '#default_value' => variable_get('measuremail_subscription_message_update', MEASUREMAIL_SUBSCRIPTION_MESSAGE_UPDATE),
    '#description' => t('Enter the message that will be displayed if this user has already subscribed.'),
    '#required' => TRUE,
  ];
  $form['measuremail_subscription_message']['measuremail_subscription_message_error'] = [
    '#type' => 'textarea',
    '#title' => t('Error'),
    '#default_value' => variable_get('measuremail_subscription_message_error', MEASUREMAIL_SUBSCRIPTION_MESSAGE_ERROR),
    '#description' => t('Enter the message that will be displayed if an error is returned by Measuremail.'),
    '#required' => TRUE,
  ];

  $form['#validate'][] = 'measuremail_subscription_admin_validate';
  $form['#submit'][] = 'measuremail_subscription_admin_submit';

  return system_settings_form($form);
}

/**
 * Custom form validation.
 *
 * @see measuremail_subscription_admin_submit()
 */
function measuremail_subscription_admin_validate($form, &$form_state) {
  //TODO: find a way to validate the input for fields and other values.

  if ($form_state['values']['measuremail_subscription_callback_type'] == 'newpage'){
    if (strlen($form_state['values']['measuremail_subscription_callback_url']) == 0) {
      form_set_error('measuremail_subscription_callback_url', t('Please define a callback URL.'));
    }
  }
}

/**
 * Custom submit function.
 *
 * @see measuremail_subscription_admin_validate()
 */
function measuremail_subscription_admin_submit($form, &$form_state) {
  $rebuild = FALSE;

  // Remove callback url value, if type different from "newpage".
  if ($form_state['values']['measuremail_subscription_callback_type'] != 'newpage'){
    $form_state['values']['measuremail_subscription_callback_url'] = '';
  }

  // Flush menu cache if the path has been changed.
  if (variable_get('measuremail_subscription_page_url', '') != $form_state['values']['measuremail_subscription_page_url']) {
    // Set the variable prior to flushing the cache otherwise, the path is still
    // unknown.
    variable_set('measuremail_subscription_page_url', $form_state['values']['measuremail_subscription_page_url']);
    $rebuild = TRUE;
  }
  if (variable_get('measuremail_subscription_callback_url', '') != $form_state['values']['measuremail_subscription_callback_url']) {
    // Set the variable prior to flushing the cache otherwise, the path is still
    // unknown.
    variable_set('measuremail_subscription_callback_url', $form_state['values']['measuremail_subscription_callback_url']);
    $rebuild = TRUE;
  }
  if ($rebuild) {
    menu_rebuild();
  }
}
